﻿using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Threading.Tasks;
using AsyncWcfService.DataAccess;
using AsyncWcfService.Entities;

namespace AsyncWcfService.BusinessLogic
{
    public class AsyncBusinessLogic
    {
        public async Task<decimal> CalculateOrderCostAsync(int accountId, List<OrderItem> orderItems)
        {
            var accountDiscountsTask = GetAccountDiscountsAsync(accountId);
            var orderItemCostTasks = orderItems
                .Select(x => CalculateItemCostAsync(x, accountDiscountsTask))
                .ToList();

            await Task.WhenAll(orderItemCostTasks).ConfigureAwait(false);

            return orderItemCostTasks.Sum(x => x.Result);
        }

        private static async Task<List<ItemDiscount>> GetAccountDiscountsAsync(int accountId)
        {
            using (var db = new EntityDbContext())
            {
                return await db.ItemDiscounts
                    .Where(x => x.AccountId == accountId)
                    .ToListAsync();
            }
        }

        public async Task<decimal> CalculateItemCostAsync(OrderItem orderItem,
            Task<List<ItemDiscount>> accountDiscountsTask)
        {
            Item item;
            using (var db = new EntityDbContext())
            {
                item = await db.Items.SingleAsync(x => x.Id == orderItem.ItemId).ConfigureAwait(false);
            }

            var accountDiscounts = await accountDiscountsTask.ConfigureAwait(false);
            var discounts = accountDiscounts
                .Where(x => x.ItemId == item.Id)
                .Select(x => x.DiscountAmount)
                .ToList();

            decimal discount = discounts.Any() ? discounts.Max() : 0m;

            return orderItem.Quantity * (item.Cost - discount);
        }
    }
}
