﻿using System.Collections.Generic;
using System.Linq;
using AsyncWcfService.DataAccess;
using AsyncWcfService.Entities;

namespace AsyncWcfService.BusinessLogic
{
    public class SyncBusinessLogic
    {
        public decimal CalculateOrderCost(int accountId, List<OrderItem> orderItems)
        {
            var accountDiscounts = GetAccountDiscounts(accountId);
            var orderItemCosts = orderItems
                .Select(x => CalculateItemCost(x, accountDiscounts))
                .ToList();

            return orderItemCosts.Sum();
        }

        private static List<ItemDiscount> GetAccountDiscounts(int accountId)
        {
            using (var db = new EntityDbContext())
            {
                return db.ItemDiscounts
                    .Where(x => x.AccountId == accountId)
                    .ToList();
            }
        }

        private decimal CalculateItemCost(OrderItem orderItem, List<ItemDiscount> accountDiscounts)
        {
            Item item;
            using (var db = new EntityDbContext())
            {
                item = db.Items.Single(x => x.Id == orderItem.ItemId);
            }

            var discounts = accountDiscounts
                .Where(x => x.ItemId == item.Id)
                .Select(x => x.DiscountAmount)
                .ToList();

            decimal discount = discounts.Any() ? discounts.Max() : 0m;

            return orderItem.Quantity * (item.Cost - discount);
        }
    }
}